@@ -239,6 +239,7 @@ def _licenses_data_from_match(
         result['short_name'] = lic.short_name
         result['category'] = lic.category
         result['is_exception'] = lic.is_exception
+        result['is_unknown'] = lic.is_unknown
         result['owner'] = lic.owner
         result['homepage_url'] = lic.homepage_url
         result['text_url'] = lic.text_urls[0] if lic.text_urls else ''
@@ -265,11 +266,13 @@ def _licenses_data_from_match(
         matched_rule['identifier'] = match.rule.identifier
         matched_rule['license_expression'] = match.rule.license_expression
         matched_rule['licenses'] = match.rule.license_keys()
+        matched_rule['referenced_filenames'] = match.rule.referenced_filenames
         matched_rule['is_license_text'] = match.rule.is_license_text
         matched_rule['is_license_notice'] = match.rule.is_license_notice
         matched_rule['is_license_reference'] = match.rule.is_license_reference
         matched_rule['is_license_tag'] = match.rule.is_license_tag
         matched_rule['is_license_intro'] = match.rule.is_license_intro
+        matched_rule['has_unknown'] = match.rule.has_unknown
         matched_rule['matcher'] = match.matcher
         matched_rule['rule_length'] = match.rule.length
         matched_rule['matched_length'] = match.len()
